'use strict';

exports.__esModule = true;

var _ora = require('ora');

var _ora2 = _interopRequireDefault(_ora);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _exec = require('@skpm/internal-utils/exec');

var _skpmConfig = require('@skpm/internal-utils/skpm-config');

var _skpmConfig2 = _interopRequireDefault(_skpmConfig);

var _auth = require('../utils/auth');

var _auth2 = _interopRequireDefault(_auth);

var _registry = require('../utils/registry');

var registry = _interopRequireWildcard(_registry);

var _asyncCommand = require('../utils/async-command');

var _asyncCommand2 = _interopRequireDefault(_asyncCommand);

var _utils = require('../utils');

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

exports.default = (0, _asyncCommand2.default)({
  command: 'publish <bump>',

  desc: 'Publish a new version of the plugin.',

  builder: {},

  handler(argv) {
    return _asyncToGenerator(function* () {
      let packageJSON;
      try {
        packageJSON = require(_path2.default.join(process.cwd(), 'package.json'));
      } catch (err) {
        (0, _utils.error)(`Error while reading the package.json file`);
        throw err;
      }

      const skpmConfig = (0, _skpmConfig2.default)(packageJSON);

      const spinner = (0, _ora2.default)({
        text: `Getting the token`,
        color: 'magenta'
      }).start();

      let token;
      if (!argv.skipRelease || !argv.skipRegistry) {
        token = yield yield _auth2.default.getToken();
      }

      // let tag = skpmConfig.version

      // if (argv.bump) {
      //   spinner.text = 'Bumping package.json version and creating git tag'
      //   const { stdout } = await exec(
      //     `npm version ${argv.bump} -m "Publish %s release :rocket:"`
      //   )
      //   tag = stdout.trim()
      // }

      // spinner.text = 'Pushing the changes to Github'

      // await exec(`git push origin HEAD`)

      // if (argv.bump) {
      //   await exec(`git push -f origin HEAD ${tag}`)
      // }
      let script = (packageJSON.scripts || {}).prepublish && 'prepublish';
      if (!script) {
        script = (packageJSON.scripts || {}).prepare && 'prepare';
      }
      if (!script) {
        script = (packageJSON.scripts || {}).build && 'build';
      }

      if (script) {
        spinner.text = 'Building the plugin';
        yield (0, _exec.exec)(`NODE_ENV=production npm run ${script}`);
      }

      spinner.text = 'Validating the manifest';

      const manifest = require(_path2.default.join(process.cwd(), skpmConfig.main, '/Contents/Sketch/manifest.json'));

      const uploadURL = yield registry.getUploadURL(token, manifest);

      spinner.text = 'Zipping the plugin';
      const tempZip = `${Date.now()}.zip`;
      yield (0, _exec.exec)(`zip -r ${tempZip} '${skpmConfig.main}' -x '*.DS_Store'`, {
        maxBuffer: 2000 * 1024
      });

      spinner.text = 'Uploading the zip asset';
      yield registry.uploadZip(uploadURL, tempZip);

      spinner.text = 'Removing the temporary zip';
      yield (0, _exec.exec)(`rm -f ${tempZip}`);

      spinner.succeed('Plugin uploaded! It is being processed. You can check the process status with `skpm process-status`');
      console.log(`${skpmConfig.name}@${manifest.version}`);
    })();
  }
});