'use strict';

exports.__esModule = true;

var _keytar = require('keytar');

var _keytar2 = _interopRequireDefault(_keytar);

var _ = require('./');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const tokenName = 'Github.com API Token';

exports.default = {
  // Get the Github API token from the keychain.
  getToken() {
    if (process.env.GITHUB_ACCESS_TOKEN) {
      return Promise.resolve(process.env.GITHUB_ACCESS_TOKEN);
    }
    return _keytar2.default.findPassword(tokenName).then(token => {
      if (!token) {
        (0, _.error)('No Github API token found in keychain\n' + 'skpm needs a GitHub token to be able to publish releases.\nThe token needs the `repo` permissions in order to create releases.\n\n' + 'Run `skpm login <token>` or set the `GITHUB_ACCESS_TOKEN` environment variable.', 1);
      }

      return token;
    });
  },
  // Save the given token to the keychain.
  //
  // token - A string token to save.
  saveToken(token) {
    _keytar2.default.setPassword(tokenName, 'github.com', token);
  },

  deleteToken() {
    _keytar2.default.deletePassword(tokenName, 'github.com');
  }
};